function [mappedX] = vl_fisher_encoder_encode(dict, X)

N_INSTANCE = size(X,1);

N_PATCH_SIZE = dict.N_PATCH_SIZE;
fprintf('use dictonary: patch size %d trained instance %d \n', N_PATCH_SIZE, dict.N_INSTANCE)


t0 = tic;
[patchX, ~, patchIdx] = extract_patches(N_PATCH_SIZE, X);
fprintf('[extract_patches] instance %d time %.2f\n', size(X,1), toc(t0));

assert(length(unique(patchIdx)) == N_INSTANCE)

N_ENCODING_DIM = size(dict.means_mag,1)*2*size(dict.means_mag,2);

mappedX = zeros(N_INSTANCE, N_ENCODING_DIM*2);


if ~isfield(dict, 'USE_DIFF_PATCH')
  useDiffSeq = false;
else
  useDiffSeq = dict.USE_DIFF_PATCH;
end

t0 = tic;
for i = [1:N_INSTANCE]
  rr = find(patchIdx==i);
  x = patchX(rr,:);

  if useDiffSeq
    patchX1 = transform_vector(x,'abs_diff');
    % patchX1 = abs(patchX1); % 777
  else
    patchX1 = transform_vector(x,'abs');
  end

  enc = vl_fisher(patchX1', dict.means_mag, dict.covariances_mag, dict.priors_mag, 'verbose');
  assert(length(enc) == size(dict.means_mag,1)*2*size(dict.means_mag,2))
  assert(length(enc) <= N_ENCODING_DIM)
  mappedX(i,1:length(enc)) = enc';


  if useDiffSeq
    patchX2 = transform_vector(x,'angle_diff');
    % patchX2 = abs(patchX2); % 777
  else
    patchX2 = transform_vector(x,'angle');
  end

  enc = vl_fisher(patchX2', dict.means_ang, dict.covariances_ang, dict.priors_ang, 'verbose');
  assert(length(enc) == size(dict.means_ang,1)*2*size(dict.means_ang,2))
  assert(length(enc) <= N_ENCODING_DIM)
  mappedX(i,(N_ENCODING_DIM+1):(N_ENCODING_DIM+length(enc))) = enc';

end